/* add-tag.c
 *
 ****************************************************************
 * Copyright (C) 2001, 2002, 2003  Tom Lord
 *
 * See the file "COPYING" for further information about
 * the copyright and warranty status of this work.
 */


#include "config-options.h"
#include "hackerlab/cmd/main.h"
#include "tla/libarch/changelogs.h"
#include "tla/libarch/inv-tags.h"
#include "tla/libarch/cmd.h"
#include "tla/libarch/cmd-add-tag.h"



static t_uchar * usage = "[options] file ...";
static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
                                   "\n"
                                   "Copyright 2001, 2002, 2003 Tom Lord\n"
                                   "\n"
                                   "This is free software; see the source for copying conditions.\n"
                                   "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n"
                                   "PARTICULAR PURPOSE.\n"
                                   "\n"
                                   "Report bugs to " cfg__tla_bug_mail ".\n"
                                   "\n");

#define OPTS(OP, OP2) \
  OP (opt_help_msg, "h", "help", 0, \
      "Display a help message and exit.") \
  OP (opt_long_help, "H", 0, 0, \
      "Display a verbose help message and exit.") \
  OP (opt_version, "V", "version", 0, \
      "Display a release identifier string") \
  OP2 (opt_version, 0, 0, 0, "and exit.")

t_uchar arch_cmd_add_tag_help[] = ("add an explicit inventory tag\n"
                                   "Create an explicit inventory tag for FILE (which may be a\n"
                                   "regular file, symbolic link, or directory).\n");


enum options
{
  OPTS (OPT_ENUM, OPT_IGN)
};

static struct opt_desc opts[] =
{
  OPTS (OPT_DESC, OPT_DESC)
    {-1, 0, 0, 0, 0}
};



int
arch_cmd_add_tag (t_uchar *program_name, int argc, char * argv[])
{
  int o;
  struct opt_parsed * option;

  safe_buffer_fd (1, 0, O_WRONLY, 0);

  option = 0;

  while (1)
    {
      o = opt_standard (lim_use_must_malloc, &option, opts, &argc, argv, program_name, usage, version_string, arch_cmd_add_tag_help, opt_help_msg, opt_long_help, opt_version);
      if (o == opt_none)
        break;
      switch (o)
        {
        default:
          safe_printfmt (2, "unhandled option `%s'\n", option->opt_string);
          panic ("internal error parsing arguments");

        usage_error:
          opt_usage (2, argv[0], program_name, usage, 1);
          exit (1);

          /* bogus_arg: */
          safe_printfmt (2, "ill-formed argument for `%s' (`%s')\n", option->opt_string, option->arg_string);
          goto usage_error;
        }
    }

  if (argc < 2)
    goto usage_error;

  {
    int a;

    for (a = 1; a < argc; ++a)
      {
        t_uchar * file;
        t_uchar * tagline_tag;
        t_uchar * new_explicit_tag;

        file = argv[a];

        if (!safe_access (file, F_OK))
          tagline_tag = arch_inventory_tag (arch_tagline_tagging, file);
        else
          tagline_tag = 0;

        if (!tagline_tag || !arch_tag_indicates_changelog (tagline_tag))
          new_explicit_tag = arch_generate_tag ();
        else
          new_explicit_tag = str_save (0, 2 + tagline_tag);

        arch_add_explicit_tag (file, new_explicit_tag);

        lim_free (0, new_explicit_tag);
        lim_free (0, tagline_tag);
      }
  }


  return 0;
}



/* tag: Tom Lord Wed May 14 12:24:43 2003 (add-tag.c)
 */
